/* libsyslogp/src/libsyslogp/500_syslogp.c
 *
 *  (c)2009, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv3. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



static char*
syslog_parse_pri(struct syslog_msg* msg, char* str)
{
    char* rd;
    unsigned int pri = 0;

    rd = str;
    if(*rd++ != '<') goto error_out;

    if(*rd == '0') {
        if(rd[1] != '>') goto error_out;
    } else {
        do {
            if(*rd < '0' || *rd > '9') goto error_out;
            pri *= 10;
            pri += *rd - '0';
            ++rd;
        }while(*rd != '>');
        if(rd - str > 4) goto error_out;
    }

    switch(pri & 7) {
    case 0:     msg->level = LOG_EMERG; break;
    case 1:     msg->level = LOG_ALERT; break;
    case 2:     msg->level = LOG_CRIT; break;
    case 3:     msg->level = LOG_ERR; break;
    case 4:     msg->level = LOG_WARNING; break;
    case 5:     msg->level = LOG_NOTICE; break;
    case 6:     msg->level = LOG_INFO; break;
    case 7:     msg->level = LOG_DEBUG; break;
    }

    switch(pri >> 3) {
    case 0:     msg->facility = LOG_KERN; break;
    case 1:     msg->facility = LOG_USER; break;
    case 2:     msg->facility = LOG_MAIL; break;
    case 3:     msg->facility = LOG_DAEMON; break;
    case 4:     msg->facility = LOG_AUTHPRIV; break;
    case 5:     msg->facility = LOG_SYSLOG; break;
    case 6:     msg->facility = LOG_LPR; break;
    case 7:     msg->facility = LOG_NEWS; break;
    case 8:     msg->facility = LOG_UUCP; break;
    case 9:     msg->facility = LOG_CRON; break;
    case 10:    msg->facility = LOG_AUTHPRIV; break;
    case 11:    msg->facility = LOG_FTP; break;
    case 12:    msg->facility = LOG_CRON; break;
    case 13:    msg->facility = LOG_SYSLOG; break;
    case 14:    msg->facility = LOG_SYSLOG; break;
    case 15:    msg->facility = LOG_CRON; break;
    case 16:    msg->facility = LOG_LOCAL0; break;
    case 17:    msg->facility = LOG_LOCAL1; break;
    case 18:    msg->facility = LOG_LOCAL2; break;
    case 19:    msg->facility = LOG_LOCAL3; break;
    case 20:    msg->facility = LOG_LOCAL4; break;
    case 21:    msg->facility = LOG_LOCAL5; break;
    case 22:    msg->facility = LOG_LOCAL6; break;
    case 23:    msg->facility = LOG_LOCAL7; break;
    default:    goto error_out;
    }

    return ++rd;

  error_out:
    msg->level = LOG_NOTICE;
    msg->facility = LOG_USER;
    return str;
}



static int
syslog_parse_month(const char* str, int* month)
{
    if(!strncmp(str, "Jan", 3)) *month = 1;
    else if(!strncmp(str, "Feb", 3)) *month = 2;
    else if(!strncmp(str, "Mar", 3)) *month = 3;
    else if(!strncmp(str, "Apr", 3)) *month = 4;
    else if(!strncmp(str, "May", 3)) *month = 5;
    else if(!strncmp(str, "Jun", 3)) *month = 6;
    else if(!strncmp(str, "Jul", 3)) *month = 7;
    else if(!strncmp(str, "Aug", 3)) *month = 8;
    else if(!strncmp(str, "Sep", 3)) *month = 9;
    else if(!strncmp(str, "Oct", 3)) *month = 10;
    else if(!strncmp(str, "Nov", 3)) *month = 11;
    else if(!strncmp(str, "Dec", 3)) *month = 12;
    else return -1;

    return 0;
}



static int
syslog_parse_implied_year = 0;



void
syslog_parse_set_implied_year(int year)
{
    syslog_parse_implied_year = year;
}



static char*
syslog_parse_date(struct syslog_msg* msg, char* str)
{
    int seen_t, len, year, month, day, hour, min, sec;
    char* p;
    time_t t;
    struct tm* tm;

    len = strlen(str);

    /* skip leading whitespace */
    for(; *str == ' '; ++str) { }

    /* check for an ISO8601 date and time (don't just try parsing the string as
     * something starting e.g. "2009 " would parse OK, but be wrong) */
    seen_t = 0;
    for(p = str; *p != ' '; ++p) {
        switch(*p) {
        case 'T':
            if(seen_t) goto not_iso;
            seen_t = 1;
            break;

        case '0' ... '9':
        case '-':
        case ':':
        case '.':
        case 'Z':
        case '+':
            break;

        default:
            goto not_iso;
        }
    }
    if(!seen_t) goto not_iso;

    *p = 0;
    if(iso8601_parse(str, &msg->timestamp, 0, 0)) {
        *p = ' ';
        goto not_iso;
    }
    return p + 1;

    /* fall back to older BSD-style or similar timestamps */
  not_iso:
    if(len >= 21 && str[3] == ' ' && str[6] == ' ' && str[11] == ' ' &&
        str[14] == ':' && str[17] == ':')
    {
        /* PIX timestamp, expected format: MMM DD YYYY HH:MM:SS: */

        if(syslog_parse_month(str, &month)) goto not_pix;

        day = strtol(str + 4, &p, 10);
        if(*p != ' ') goto not_pix;

        year = strtol(str + 7, &p, 10);
        if(*p != ' ') goto not_pix;

        hour = strtol(str + 12, &p, 10);
        if(*p != ':') goto not_pix;

        min = strtol(str + 15, &p, 10);
        if(*p != ':') goto not_pix;

        sec = strtol(str + 18, &p, 10);
        if(*p != ' ' && *p != ':') goto not_pix;

        if(iso8601_from_cal(&msg->timestamp, year, month, day) ||
            iso8601_from_clocktime(&msg->timestamp, hour, min, sec))
        {
            goto not_pix;
        }

        return str + ((str[20] == ' ') ? 21 : 20);
    }

  not_pix:
    if(len >= 21 && str[3] == ' ' && str[6] == ' ' && str[9] == ':' &&
        str[12] == ':' && str[15] == ' ' && str[20] == ' ')
    {
        /* Linksys timestamp, expected format: MMM DD HH:MM:SS YYYY */

        if(syslog_parse_month(str, &month)) goto not_linksys;

        day = strtol(str + 4, &p, 10);
        if(*p != ' ') goto not_linksys;

        hour = strtol(str + 7, &p, 10);
        if(*p != ':') goto not_linksys;

        min = strtol(str + 10, &p, 10);
        if(*p != ':') goto not_linksys;

        sec = strtol(str + 13, &p, 10);
        if(*p != ' ') goto not_linksys;

        year = strtol(str + 16, &p, 10);
        if(*p != ' ' || year < 1970 || year > 2100) goto not_linksys;

        if(iso8601_from_cal(&msg->timestamp, year, month, day) ||
            iso8601_from_clocktime(&msg->timestamp, hour, min, sec))
        {
            goto not_linksys;
        }

        return str + 21;
    }

  not_linksys:
    if(len >= 15 && str[3] == ' ' && str[6] == ' ' && str[9] == ':' &&
        str[12] == ':')
    {
        /* RFC 3164 timestamp, expected format: MMM DD HH:MM:SS ... */

        if(syslog_parse_month(str, &month)) goto not_bsd;

        day = strtol(str + 4, &p, 10);
        if(*p != ' ') goto not_bsd;

        hour = strtol(str + 7, &p, 10);
        if(*p != ':') goto not_bsd;

        min = strtol(str + 10, &p, 10);
        if(*p != ':') goto not_bsd;

        sec = strtol(str + 13, &p, 10);
        if(*p != ' ' && *p != ':' && *p != '.') goto not_bsd;

        /* get year */
        if(syslog_parse_implied_year) {
            year = syslog_parse_implied_year;
        } else {
            time(&t);
            tm = gmtime(&t);
            year = tm->tm_year + 1900;
        }

        if(iso8601_from_cal(&msg->timestamp, year, month, day) ||
            iso8601_from_clocktime(&msg->timestamp, hour, min, sec))
        {
            goto not_bsd;
        }

        /* check for fractional seconds */
        p = str + 15;
        if(*p != '.') return str + 16; /* skip trailing ':' or space */

        sec = 0;
        min = 0;
        ++p;
        do {
            if(*p < '0' || *p > '9') return str + 15;
            ++min;
            sec *= 10;
            sec += *p - '0';
            ++p;
        }while(*p && *p != ' ');

        if(min > 9) return str + 15;
        for(msg->timestamp.nsec = sec; min < 9; ++min) {
            msg->timestamp.nsec *= 10;
        }

        return p;
    }

  not_bsd:
    return str;
}



static char*
syslog_parse_spacefield(char** ptr, char* str)
{
    /* skip whitespace */
    for(; *str == ' '; ++str) { }
    if(!*str) return str;

    /* store pointer to string */
    *ptr = str;

    /* move to end of string */
    for(; *str != ' '; ++str) {
        if(!*str) return str;
    }

    /* null terminate and move to next field */
    *str++ = 0;
    return str;
}



static char*
syslog_parse_tag_id(struct syslog_msg* msg, char* str)
{
    char* p;

    /* skip whitespace */
    for(; *str == ' '; ++str) { }
    if(!*str) return str;

    /* grab tag */
    msg->tag = str;

    /* search for tag delimiter */
    for(;; ++str) {
        switch(*str) {
        case ':':
        case '.':
        case '[':
        case '{':
        case '<':
            goto found_id;

        case ' ':
        case 0:
            goto after_id;
        }
    }

    /* try to extract an ID */
  found_id:
    p = 0;
    switch(*str) {
    case ':':
        if(str[1] == ' ') goto after_id;
        /* fall through */
    case '.':
        *str++ = 0;
        msg->id = str;
        for(; *str && *str != ':' && *str != ' '; ++str) { }
        goto after_id;

    case '[':
        p = strchr(str, ']');
        break;

    case '{':
        p = strchr(str, '}');
        break;

    case '<':
        p = strchr(str, '>');
        break;
    }
    if(!p) return str;

    *str++ = 0;
    msg->id = str;
    *p = 0;
    str = p + 1;
    if(*str == ':') ++str;
    if(*str == ' ') ++str;
    return str;

  after_id:
    if(!*str) return str;
    *str++ = 0;
    if(*str == ' ') ++str;
    return str;
}



void
syslog_parse(struct syslog_msg* msg, char* str)
{
    int version = 0, len;

    memset(msg, 0, sizeof(struct syslog_msg));
    str = syslog_parse_pri(msg, str);

    /* check for an RFC5424 version 1 message */
    if(!strncmp(str, "1 ", 2)) {
        str += 2;
        version = 1;
    }

    str = syslog_parse_date(msg, str);
    str = syslog_parse_spacefield(&msg->hostname, str);

    if(version == 1) {
        str = syslog_parse_spacefield(&msg->tag, str);
        str = syslog_parse_spacefield(&msg->id, str);
    } else {
        str = syslog_parse_tag_id(msg, str);
    }

    /* get message part, removing any trailing \n */
    len = strlen(str);
    if(len && str[len - 1] == '\n') str[len - 1] = 0;
    msg->msg = str;
}



const char*
syslog_level_name(int level)
{
    switch(level) {
    case LOG_EMERG:             return "emergency";
    case LOG_ALERT:             return "alert";
    case LOG_CRIT:              return "critical";
    case LOG_ERR:               return "error";
    case LOG_WARNING:           return "warning";
    case LOG_NOTICE:            return "notice";
    case LOG_INFO:              return "info";
    case LOG_DEBUG:             return "debug";
    }
    return 0;
}



const char*
syslog_facility_name(int facility)
{
    switch(facility) {
    case LOG_AUTH:
    case LOG_AUTHPRIV:          return "auth";
    case LOG_CRON:              return "cron";
    case LOG_DAEMON:            return "daemon";
    case LOG_FTP:               return "ftp";
    case LOG_KERN:              return "kernel";
    case LOG_LOCAL0:            return "local0";
    case LOG_LOCAL1:            return "local1";
    case LOG_LOCAL2:            return "local2";
    case LOG_LOCAL3:            return "local3";
    case LOG_LOCAL4:            return "local4";
    case LOG_LOCAL5:            return "local5";
    case LOG_LOCAL6:            return "local6";
    case LOG_LOCAL7:            return "local7";
    case LOG_LPR:               return "printer";
    case LOG_MAIL:              return "mail";
    case LOG_NEWS:              return "news";
    case LOG_SYSLOG:            return "syslog";
    case LOG_USER:              return "user";
    case LOG_UUCP:              return "uucp";
    }
    return 0;
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
