/* libsyslogp/src/libsyslogp/500_syslogp.h
 *
 *  (c)2009, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv3. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



/*! \defgroup syslogp syslog message parser

This set of routines allows parsing of a syslog message and processing of the
result. \ref syslog_parse() is called on the raw message data and the component
parts of the message are then stored in a struct \ref syslog_msg.

Note: BSD-syslog style messages don't generally contain a year in the timestamp
field. This means that \ref syslog_parse() calls \c time(3) to find the current
year whenever it parses such a message. This is expensive and, potentially,
incaccurate. You can set the year implied in such messages with
\ref syslog_parse_set_implied_year().

*/
/*!@{*/



/*! \brief Parsed syslog message.

The component parts of an RFC3164 syslog message. The MSGID and SD (structured
data) parts of an RFC5424 syslog message will not be parsed and will be
considered part of the \a msg text.

*/
struct syslog_msg {
    /*! \brief Message severity level.

    This is one of the \c LOG_INFO/LOG_WARNING etc. types from \c syslog(3). It
    can be turned into a friendly name via \ref syslog_level_name(). It will be
    set to \c LOG_NOTICE if the priority field could not be recovered from the
    original message.

    */
    int level;

    /*! \brief Message facility.

    This is one of the \c LOG_DAEMON/LOG_LOCAL0 etc. types from \c syslog(3). It
    can be turned into a friendly name via \ref syslog_facility_name(). It will
    be set to \c LOG_USER if the priority field could not be recovered from the
    original message.

    */
    int facility;

    /*! \brief Timestamp.

    The timestamp of the message. It will be all 0 if the timestamp could not
    be recovered from the original message.

    */
    struct iso8601_date timestamp;

    /*! \brief Hostname.

    The hostname as specified by the producing application. If this could not be
    recovered, it will be set to 0.

    */
    char* hostname;

    /*! \brief Producing application name (tag).

    The name (or tag) of the application which produced the original message. If
    this could not be recovered, it will be set to 0.

    */
    char* tag;

    /*! \brief Producing application instance ID.

    Many (but not all) RFC3164 messages place an application instance ID in
    square brackets immediately after the tag. RFC5424 has a field specifically
    for this purpose. If discovered, this will point to the data in that field.
    If not, it will be 0.

    */
    char* id;

    /*! \brief Message text.

    This points to the remaining message text once the header and tag have been
    stripped off. It may be 0 or a pointer to an empty string if the message was
    truly empty.

    */
    char* msg;
};



/*! \brief Parse a syslog message.

\param[out] msg Structure into which parsed message data is written.
\param str Pointer to message string. Null terminated. Will be altered!

This function parses the syslog message written into \a str, which is a null
terminated C string containing the syslog message. \a str will be altered by
this process (e.g. some characters will be overwritten with ASCII NULs). The
parsed message data is written into \a msg.

\a msg contains several string pointers (such as \a tag, \a id and \a msg).
After parsing, these will be set to be pointers into \a str. Each component
will be null-terminated (which is why \a str will be altered).

*/
void syslog_parse(struct syslog_msg* msg, char* str)
#ifndef DOXYGEN
    __attribute__((nonnull))
#endif
;



/*! \brief Set year implied in BSD-style timestamps.

\param year Year to assume (or 0 to query the system clock).

BSD-style timestamps do not contain the current year. libsyslogp will thus query
the system clock when it is trying to parse such a timestamp. This is expensive
and, potentially, inaccurate. This function can be used to set the implied year.

*/
void syslog_parse_set_implied_year(int year);



/*! \brief Get friendly name for syslog level.

\param level \c syslog(3) log level constant.
\returns Pointer to string constant.
\retval 0 if \a level is invalid.

This function interprets a \c syslog(3) log level constant (such as \c LOG_INFO
or \c LOG_WARNING) and returns a friendly string ("info", "warning"). If the
value of \a level is invalid, it returns a null pointer.

*/
const char* syslog_level_name(int level);



/*! \brief Get friendly name for syslog facility.

\param facility \c syslog(3) log facility constant.
\returns Pointer to string constant.
\retval 0 if \a facility is invalid.

This function interprets a \c syslog(3) log facility constant (such as
\c LOG_DAEMON or \c LOG_USER) and returns a friendly string ("daemon", "user").
If the value of \a facility is invalid, it returns a null pointer.

*/
const char* syslog_facility_name(int facility);



/*!@}*/
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
